/**
 * @fileOverview
 * @name network.js
 * @author Wagner Müller Gegler, Roberto Oppermann Cordoni
 */

/**
 * @typedef {Object} NetworkController~InterfaceParams - Configuration parameters for a network interface.
 * @property {String} bootproto - [|none] The protocol to be used at boot time.
 * @property {String} dns1 - Name server IP address - option one.
 * @property {String} dns2 - Name server IP address - option two.
 * @property {String} gateway - Gateway device IP address.
 * @property {String} ipaddr - IP address for the interface.
 * @property {String} mode - [static|disabled|dhcp] Operation mode for the interface.
 * @property {String} name - Interface name.
 * @property {String} netmask - Network mask of this network, in dot-decimal notation.
 * @property {String} onboot - [yes|] Enable/Disable activation of interface at boot time.
 */

/**
 * @class NetworkController
 * @param {Object} $http - AngularJS service for HTTP communication.
 * @param {Object} $log - AngularJS service for logging.
 * @property {boolean[]} valid_ipaddr - Information list about the validity of the IP address field, for each network interface.
 * @property {boolean[]} valid_Netmask - Information list about the validity of the netmask address field, for each network interface.
 * @property {boolean[]} valid_gateway - Information list about the validity of the Gateway address field, for each network interface.
 * @property {boolean[]} valid_dns1 - Information list about the validity of the DNS1 address field, for each network interface.
 * @property {boolean[]} valid_dns1 - Information list about the validity of the DNS2 address field, for each network interface.
 * @property {boolean[]} loading - Information list about the validity of the DNS2 address field, for each network interface.
 * @property {NetworkController~InterfaceParams[]} interfaces - List of network parameters.
 */

angular.module('ag561app').controller('NetworkController', ['$http', '$log', '$location',
                                   function($http, $log, $location) {
  var url_network = '/api/config/network';
  var ctrl = this;
  ctrl.alert = false;
  ctrl.loading = false;
  ctrl.invalid_config = false;
  ctrl.invalid_ipaddr = false;
  ctrl.invalid_netmask = false;
  ctrl.invalid_gateway = false;
  ctrl.invalid_hostname = false;
  ctrl.invalid_dns1 = false;
  ctrl.invalid_dns2 = false;
  ctrl.invalid_http = false;
  ctrl.invalid_https = false;
  ctrl.invalid_ssh = false;
  ctrl.invalid_telnet = false;
  ctrl.invalid_rocommunity = false;
  ctrl.invalid_rwcommunity = false;
  ctrl.invalid_alias_name = false;
  ctrl.invalid_alias_addr = false;
  ctrl.invalid_alias_netmask = false;
  ctrl.show_snmp = false;
  ctrl.show_ip = true;
  ctrl.show_switch = false;
  ctrl.show_cpu = false;
  ctrl.show_aliases = false;
  ctrl.alias_indextoedit = -1;
  ctrl.error_msg = "Invalid configuration.";
  ctrl.ipformat = new RegExp("^(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)$");
  ctrl.subnet_format = new RegExp("^(((255\.){3}(255|254|252|248|240|224|192|128|0+))|((255\.){2}(255|254|252|248|240|224|192|128|0+)\.0)|((255\.)(255|254|252|248|240|224|192|128|0+)(\.0+){2})|((255|254|252|248|240|224|192|128|0+)(\.0+){3}))$");

  /**
   * Clear alert message from web page, but do this only
   * if we aren't with some error. We just want to erase
   * the message of danger if all the variables are correct.
   *
   * @function clear_alert_messages
   * @memberof NetworkController
   */
  ctrl.clear_alert_messages = function() {
    if (!ctrl.invalid_config) {
      ctrl.alert_danger = false;
      ctrl.alert_success = false;
    }
    if (ctrl.alert) {
      ctrl.alert_danger = true;
      ctrl.alert = false;
    }
  };

  /**
   * Get network config and DNS parameters.
   * @function get
   * @memberof NetworkController
   */
  ctrl.get = function() {
    $http({
        method: 'GET',
        url: url_network
        }).then(function(response) {
            // First function handles success
            ctrl.ip = response.data.ip;
            ctrl.snmp = response.data.snmp;
            ctrl.switch = response.data.switch;
            ctrl.services = response.data.services;

            // Check non mandatory parameters
            if (!ctrl.switch['monitor_port'])
              ctrl.switch.monitor_port = 'none';

            // Set some variables
            ctrl.switch.port1.invalid_allowed_vids = false;
            ctrl.switch.port2.invalid_allowed_vids = false;
            update_interfaces_metadata();

        }, function(response) {
            // Second function handles error
            ctrl.error = response.data;
        });
    }

  /**
   * Save the new configuration.
   * @function save
   * @memberof NetworkController
   */
  ctrl.save = function() {

    // Remove variables that cannot go to API
    delete ctrl.switch.port1.invalid_allowed_vids;
    delete ctrl.switch.port2.invalid_allowed_vids;
    delete ctrl.switch.cpu.invalid_allowed_vids;

    var data = {
      'services'  : ctrl.services,
      'switch'    : ctrl.switch,
      'snmp'      : ctrl.snmp,
      'ip'        : ctrl.ip
    }

    ctrl.loading = true;
    // PUT data to API
    $http({
      method: 'PUT',
      url: url_network,
      data: data
      }).then(function(response) {
          // First function handles success
          ctrl.loading = false;
          ctrl.alert_success = true;
          ctrl.get();
      }, function(response) {
          // Second function handles error
          ctrl.error = response.data;
          ctrl.alert_danger = true;
      });
  };

  /**
   * Add new alias to object 'ctrl.ip.aliases'
   *
   * @function alias_add
   * @memberof NetworkController
   */
   ctrl.alias_add = function() {

    if (alias_limit_was_reached()) {
      ctrl.error_msg = "Aliases limit was reached (16)";
      ctrl.alert = true;
      return;
    }

    var alias = {'name': '',
                'addr': '',
                'mask': ''
    };

    ctrl.invalid_alias_name = true;
    ctrl.invalid_alias_addr = true;
    ctrl.invalid_alias_netmask = true;

    ctrl.ip.aliases.push(alias);
    ctrl.alias_indextoedit = ctrl.ip.aliases.indexOf(alias);
    update_interfaces_metadata();
    __check_invalid_config();
  };

  /**
   * Remove specific alias from main object 'ctrl.ip.aliases'
   *
   * @function alias_delete
   * @memberof NetworkController
   */
   ctrl.alias_delete = function(index) {
    ctrl.ip.aliases.splice(index, 1);
    ctrl.invalid_alias_name = false;
    ctrl.invalid_alias_addr = false;
    ctrl.invalid_alias_netmask = false;
    update_interfaces_metadata();
    __check_invalid_config();
  }

  /**
   * Edit specific alias from main object 'ctrl.ip.aliases'
   *
   * @function alias_edit
   * @memberof NetworkController
   */
  ctrl.alias_edit = function(index) {
    ctrl.alias_indextoedit = index;
  }

  /**
   * Confirm alias edition
   *
   * @function alias_confirm
   * @memberof NetworkController
   */
  ctrl.alias_confirm = function() {
    ctrl.alias_indextoedit = -1;
    update_interfaces_metadata();
  }

  /**
   * Update number of aliases
   *
   * @function update_interfaces_metadata
   * @memberof NetworkController
   */
  function update_interfaces_metadata() {
    ctrl.number_of_aliases = 0;
    for (var a = 0; a < ctrl.ip.aliases.length; a++) {
      ctrl.number_of_aliases++;
    }
  }

  /**
   * Limit the number of aliases
   *
   * @function alias_limit_was_reached
   * @memberof NetworkController
   */
  function alias_limit_was_reached() {

    if (ctrl.number_of_aliases >= 16)
      return true;

    return false;
  }

  /**
   * Check if some configuration is invalid. If true, then disable the
   * button to 'save' config and show the 'danger message'.
   *
   * @function __check_invalid_config
   * @memberof NetworkController
   */
  __check_invalid_config = function() {
    if (ctrl.invalid_ipaddr || ctrl.invalid_netmask || ctrl.invalid_gateway ||
        ctrl.invalid_dns1 || ctrl.invalid_dns2 ||
        ctrl.invalid_pppoe_username || ctrl.invalid_pppoe_password ||
        ctrl.invalid_ntp || ctrl.invalid_syslog || ctrl.invalid_allowed_vids ||
        ctrl.invalid_rocommunity || ctrl.invalid_rwcommunity ||
        ctrl.invalid_alias_name || ctrl.invalid_alias_addr ||
        ctrl.invalid_alias_netmask || ctrl.invalid_http || ctrl.invalid_https ||
        ctrl.invalid_ssh || ctrl.invalid_telnet ) {
      ctrl.invalid_config = true;
      ctrl.alert_danger = true;
      ctrl.error_msg = "Invalid configuration.";
    } else {
      ctrl.invalid_config = false;
      ctrl.alert_danger = false;
    }
  };

  ctrl.validate_ip_config = function() {
      /* Check static IP address and netmask */
      ctrl.invalid_ipaddr = ctrl.ip.type == 'static' && ctrl.ip.addr && !ctrl.ip.addr.match(ctrl.ipformat);
      ctrl.invalid_netmask = ctrl.ip.type == 'static' && ctrl.ip.mask && !ctrl.ip.mask.match(ctrl.ipformat);

      /* Default Gateway is optional. Only check if it is not empty */
      ctrl.invalid_gateway = ctrl.ip.gateway && !ctrl.ip.gateway.match(ipformat);

      /* DNS Servers are optional. Only check if it is not empty */
      ctrl.invalid_dns1 = ctrl.ip.type != 'pppoe' && ctrl.ip.dns[0] && !ctrl.ip.dns[0].match(ctrl.ipformat);
      ctrl.invalid_dns2 = ctrl.ip.type != 'pppoe' && ctrl.ip.dns[1] && !ctrl.ip.dns[1].match(ctrl.ipformat);

      /* Validate PPPoE username and password */
      ctrl.invalid_pppoe_username = ctrl.ip.type == 'pppoe' && !ctrl.ip.pppoe_username.match(/^([\S]+)$/);
      ctrl.invalid_pppoe_password = ctrl.ip.type == 'pppoe' && !ctrl.ip.pppoe_password.match(/^([\S]+)$/);

      __check_invalid_config();
  };

  /* Server eg.: aligera:1234 or 192.168.2.1:1234 or 10.10.10.1 */
  var server_port_regex = /^([\w.-]+)(:[\d]+)?$/;
  var community_regex = /^[\w\-.]+$/;

  ctrl.validateNTP = function() {
    ctrl.invalid_ntp = ctrl.ip.ntp && !ctrl.ip.ntp.match(server_port_regex);
     __check_invalid_config();
  };

  ctrl.validateSyslog = function() {
    ctrl.invalid_syslog = ctrl.ip.syslog && !ctrl.ip.syslog.match(server_port_regex);
    __check_invalid_config();
  };

  ctrl.validateRoCommunity = function() {
    ctrl.invalid_rocommunity = ctrl.snmp.rocommunity && !ctrl.snmp.rocommunity.match(community_regex);
     __check_invalid_config();
  };


  ctrl.validateRwCommunity = function() {
    ctrl.invalid_rwcommunity = ctrl.snmp.rwcommunity && !ctrl.snmp.rwcommunity.match(community_regex);
     __check_invalid_config();
  };

  ctrl.validateHostname = function() {
    var regex = /^[a-zA-Z0-9][a-zA-Z0-9\.\-]*$/;

    ctrl.invalid_hostname = ctrl.ip.hostname.match(regex) ? false : true;

    __check_invalid_config();
  };

  /**
   * Check if alias ID is valid
   *
   * @function validateAliasId
   * @memberof NetworkController
   */
   ctrl.validateAliasId = function(alias, index) {
    ctrl.invalid_alias_name = true;

    if (_already_exists(ctrl.ip.aliases, alias.name, index)) {
      __check_invalid_config();
      return;
    }

    if (alias.name.length == 0) {
       __check_invalid_config();
       return;
    }

    ctrl.invalid_alias_name = false;
    __check_invalid_config();
  }

  _already_exists = (aliases, name, index) => {

    for (let v = 0; v < aliases.length; v++) {
      if ((aliases[v].name == name) && index != v)
        return true;
    }

    return false;
  }

  /**
   * Check if IP and mask alias are valid
   *
   * @function validateIpAndSubnet
   * @memberof NetworkController
   */
  ctrl.validateIpAndSubnet = function(alias) {
    ctrl.invalid_alias_addr = ((!alias.addr || alias.addr == "") ||
      (alias.addr && alias.addr.match(ctrl.ipformat))) ? false : true;

    ctrl.invalid_alias_netmask = ((!alias.mask || alias.mask == "") ||
      (alias.mask && alias.mask.match(ctrl.subnet_format))) ? false : true;

    __check_invalid_config();
  };

  /**
   * Check if port allowed VLAN IDs are valid. If not valid, sets a control variable called
   * invalid_allowed_vids, for the port passed as parameter. Also, calls __check_invalid_config()
   * function to disable the save button.
   *
   * @function validate_allowed_vids
   * @memberof NetworkController
   */
  ctrl.validate_allowed_vids = function(port) {
    port.invalid_allowed_vids = false;

    // Split array by spaces or commas
    var vids = port.allowed_vids.split(/[ ,]+/);
    for (var i = 0; i < vids.length; i++) {
      vid = parseInt(vids[i], 10);
      // Check VID range and if it doesnt have letter in it
      if (vid < 2 || vid > 4094 || vids[i].search(/[a-zA-Z]+/) >= 0)
        port.invalid_allowed_vids = true;
    }

    if (port.invalid_allowed_vids == true)
      ctrl.invalid_allowed_vids = true;
    else
      ctrl.invalid_allowed_vids = false;

    __check_invalid_config();
  };

  ctrl.validatePort = function(service, port) {
    var invalid_port = true;

    if (checkPort(service, port)) {
      invalid_port = false;
    }

    if (service === 'http')
      ctrl.invalid_http = invalid_port;
    if (service === 'https')
      ctrl.invalid_https = invalid_port;
    if (service === 'ssh')
      ctrl.invalid_ssh = invalid_port;
    if (service === 'telnet')
      ctrl.invalid_telnet = invalid_port;

    __check_invalid_config();
  };

  function checkPort(service, port) {

    for (const [ key, value ] of Object.entries(ctrl.services)) {
      if (port === value && service != key) {
        return false;
      }
    }

    if (port > 0 && port < 65536)
      return true;

    return false;
  };

  ctrl.get();

}]);
